"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_1 = require("graphql");
var Field_1 = require("../field/Field");
var structure_1 = require("../type-expression/structure");
var directive_1 = require("./directive");
var TypeExpression_1 = require("../type-expression/TypeExpression");
var InputField_1 = require("../field/InputField");
var SUBSTITUTION_PREFIX = '__GIRIN__SUBS__';
var TEMP_PLACEHOLDER = '__GIRIN__TEMP__';
function gql(strings) {
    var interpolated = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        interpolated[_i - 1] = arguments[_i];
    }
    var result = [strings[0]];
    var subsMap = {};
    for (var i = 0; i < interpolated.length; i++) {
        var item = interpolated[i];
        var name = "" + SUBSTITUTION_PREFIX + i;
        if (item instanceof Field_1.FieldMount) {
            subsMap[name] = item;
            result.push(name + ": " + TEMP_PLACEHOLDER);
        }
        else {
            subsMap[name] = item;
            result.push(name);
        }
        result.push(strings[i + 1]);
    }
    var ast = graphql_1.parse(result.join(''));
    if (ast.definitions.length > 1) {
        throw new Error('Only one type definition should be passed to gql tag');
    }
    var rootNode = ast.definitions[0];
    return new ASTParser(rootNode, subsMap);
}
exports.gql = gql;
var ASTParser = /** @class */ (function () {
    function ASTParser(rootNode, subsMap) {
        this.objectTypeMetadataConfigs = [];
        this.interfaceTypeMetadataConfigs = [];
        this.inputObjectTypeMetadataConfigs = [];
        this.fieldMetadataConfigs = [];
        this.inputFieldMetadataConfigs = [];
        this.rootNode = rootNode;
        this.subsMap = subsMap;
        this.createMetadataFromAST();
    }
    ASTParser.prototype.createMetadataFromAST = function () {
        var rootNode = this.rootNode;
        if (rootNode.kind === 'ObjectTypeDefinition') {
            this.appendObjectTypeConfig(rootNode);
        }
        else if (rootNode.kind === 'InterfaceTypeDefinition') {
            this.appendInterfaceTypeMetadataConfig(rootNode);
        }
        else if (rootNode.kind === 'InputObjectTypeDefinition') {
            this.appendInputObjectTypeMetadataConfig(rootNode);
        }
        else {
            throw new Error("Node type not supported: " + rootNode.kind);
        }
    };
    ASTParser.prototype.completeTypeExpression = function (type) {
        var subsMap = this.subsMap;
        if (type.kind === 'ListType') {
            return structure_1.List.of(this.completeTypeExpression(type.type));
        }
        else if (type.kind === 'NonNullType') {
            return structure_1.NonNull.of(this.completeTypeExpression(type.type));
        }
        else {
            var sub = subsMap[type.name.value];
            if (!sub) {
                return new TypeExpression_1.TypeExpression(type.name.value);
            }
            else if (sub instanceof TypeExpression_1.TypeExpression) {
                return sub;
            }
            return new TypeExpression_1.TypeExpression(sub);
        }
    };
    ASTParser.prototype.appendObjectTypeConfig = function (node) {
        var _this = this;
        var name = node.name, description = node.description, interfaces = node.interfaces, fields = node.fields;
        if (fields) {
            fields.forEach(function (fieldNode) { return _this.appendFieldMetadataConfig(fieldNode); });
        }
        var interfacesTypeExpressions = interfaces && interfaces.map(function (interfaceNode) { return (_this.completeTypeExpression(interfaceNode)); });
        this.objectTypeMetadataConfigs.push({
            interfaces: interfacesTypeExpressions,
            typeName: name.value,
            description: description && description.value,
            directives: node.directives && directive_1.completeDirectives(node.directives),
        });
    };
    ASTParser.prototype.appendInterfaceTypeMetadataConfig = function (node) {
        var _this = this;
        var name = node.name, description = node.description, fields = node.fields;
        if (fields) {
            fields.forEach(function (fieldNode) { return _this.appendFieldMetadataConfig(fieldNode); });
        }
        this.interfaceTypeMetadataConfigs.push({
            typeName: name.value,
            description: description && description.value,
            directives: node.directives && directive_1.completeDirectives(node.directives),
        });
    };
    ASTParser.prototype.appendInputObjectTypeMetadataConfig = function (node) {
        var _this = this;
        var name = node.name, description = node.description, fields = node.fields;
        if (fields) {
            fields.forEach(function (fieldNode) { return _this.appendInputFieldMetadataConfig(fieldNode); });
        }
        this.inputObjectTypeMetadataConfigs.push({
            typeName: name.value,
            description: description && description.value,
            directives: node.directives && directive_1.completeDirectives(node.directives),
        });
    };
    ASTParser.prototype.appendFieldMetadataConfig = function (node) {
        var _this = this;
        var name = node.name, type = node.type, description = node.description, directives = node.directives, args = node.arguments;
        var sub = this.subsMap[name.value];
        var mount;
        if (sub) {
            mount = sub;
        }
        else {
            var argumentRefs = args && args.map(function (argumentNode) { return (_this.createInputField(argumentNode)); });
            var field = new Field_1.Field({
                defaultName: name.value,
                type: this.completeTypeExpression(type),
                args: argumentRefs || [],
                description: description && description.value,
                directives: directives && directive_1.completeDirectives(directives),
            });
            mount = new Field_1.FieldMount({ mountName: name.value, field: field });
        }
        this.fieldMetadataConfigs.push(mount);
    };
    ASTParser.prototype.createInputField = function (node) {
        var name = node.name, type = node.type, description = node.description, defaultValue = node.defaultValue, directives = node.directives;
        return new InputField_1.InputField({
            defaultName: name.value,
            type: this.completeTypeExpression(type),
            directives: directives && directive_1.completeDirectives(directives),
            defaultValue: defaultValue && directive_1.completeValueNode(defaultValue),
            description: description && description.value,
        });
    };
    ASTParser.prototype.appendInputFieldMetadataConfig = function (node) {
        this.inputFieldMetadataConfigs.push(this.createInputField(node));
    };
    return ASTParser;
}());
exports.ASTParser = ASTParser;
//# sourceMappingURL=ast.js.map